# -*- coding: utf-8 -*-
from collective_folderprotection.behaviors.interfaces import IPasswordProtected
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.interfaces import IPasswordWidget
from z3c.form.util import getSpecification
from z3c.form.widget import FieldWidget
from z3c.form.browser.password import PasswordWidget
from zope.component import adapter
from zope.interface import implementer
from zope.interface import implementer_only


class IPasswordNoAutocompleteWidget(IPasswordWidget):
    """ """


@implementer_only(IPasswordNoAutocompleteWidget)
class PasswordNoAutocompleteWidget(PasswordWidget):
    """Input type password widget implementation with no autocomplete."""


@adapter(getSpecification(IPasswordProtected["passw_hash"]), IFormLayer)
@implementer(IFieldWidget)
def PasswordNoAutocompleteFieldWidget(field, request):
    return FieldWidget(field, PasswordNoAutocompleteWidget(request))



import six
from zope.interface import implementer
from zope.interface.common.interfaces import IException


class IPasswordProtectedUnauthorized(IException):
    """ """


class IDeleteProtectionException(IException):
    """
    an exception raised when a delete protected folder is attempted to be deleted
    """


class IRenameProtectionException(IException):
    """
    an exception raised when a rename protected folder is attempted to be renamed
    """


@implementer(IPasswordProtectedUnauthorized)
class PasswordProtectedUnauthorized(Exception):
    def _get_message(self):
        return self._message

    message = property(_get_message)

    def __init__(self, message=None, name=None):
        """Possible signatures:

        PasswordProtectedUnauthorized()
        PasswordProtectedUnauthorized(message)
        PasswordProtectedUnauthorized(name)
        PasswordProtectedUnauthorized(message, name)

        """

        self.name = name
        self._message = message

    def __str__(self):
        if self.message is not None:
            return self.message
        if self.name is not None:
            return "You need a password to access '%s'" % self.name
        return repr(self)

    if six.PY2:

        def __unicode__(self):
            result = self.__str__()
            if isinstance(result, unicode):  # noqa: F821
                return result
            return unicode(  # noqa: F821
                result, "ascii"
            )  # override sys.getdefaultencoding()


@implementer(IDeleteProtectionException)
class DeleteProtectionException(Exception):
    """ """


@implementer(IRenameProtectionException)
class RenameProtectionException(Exception):
    """ """



# -*- coding: utf-8 -*-
from collective_folderprotection.testing import ACCEPTANCE_TESTING
from plone.app.testing import ROBOT_TEST_LEVEL
from plone.testing import layered
import os
import robotsuite
import unittest


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, "robot")
    robot_tests = [
        os.path.join("robot", doc)
        for doc in os.listdir(robot_dir)
        if doc.endswith(".robot") and doc.startswith("test_")
    ]
    for robot_test in robot_tests:
        robottestsuite = robotsuite.RobotTestSuite(robot_test)
        robottestsuite.level = ROBOT_TEST_LEVEL
        suite.addTests([layered(robottestsuite, layer=ACCEPTANCE_TESTING)])
    return suite


# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from collective_folderprotection import _
from collective_folderprotection.behaviors.interfaces import IDeleteProtected
from plone.app.content.browser.contents.delete import (
    DeleteActionView as BaseDeleteActionView,
)
from plone.app.uuid.utils import uuidToCatalogBrain
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.component.hooks import getSite

import json


FOLDER_PROTECTED_TEMPLATE = """
<h2>This folder is protected</h2>
<div id="content-core">
<p>Items inside this folder are protected against being deleted.</p>
</div>
"""


class DeleteActionView(BaseDeleteActionView):
    def __call__(self, keep_selection_order=False):
        self.protect()
        self.errors = list()
        if self.request.form.get("render") == "yes":
            confirm_view = getMultiAdapter(
                (getSite(), self.request), name="delete_confirmation_info"
            )
            selection = self.get_selection()
            self.request.response.setHeader(
                "Content-Type", "application/json; charset=utf-8"
            )
            # Check the current folder allows to remove items
            adapter = IDeleteProtected(self.context, None)
            if adapter:
                if adapter.delete_protection:
                    self.errors.append(
                        _(
                            "This folder is protected against deleting "
                            "items inside of it."
                        )
                    )
                    return json.dumps({"html": FOLDER_PROTECTED_TEMPLATE})

            catalog = getToolByName(self.context, "portal_catalog")
            brains = catalog(UID=selection, show_inactive=True)
            items = [i.getObject() for i in brains]
            self.request.response.setHeader(
                "Content-Type", "application/json; charset=utf-8"
            )
            return json.dumps({"html": confirm_view(items)})
        else:
            context = aq_inner(self.context)
            selection = self.get_selection()

            parts = str(self.request.form.get("folder", "").lstrip("/")).split("/")
            if parts:
                parent = self.site.unrestrictedTraverse("/".join(parts[:-1]))
                self.dest = parent.restrictedTraverse(parts[-1])

            # Check the current folder allows to remove items
            adapter = IDeleteProtected(self.context, None)
            if adapter:
                if adapter.delete_protection:
                    self.errors.append(
                        _(
                            "This folder is protected against deleting "
                            "items inside of it."
                        )
                    )
                    return self.message(list())

            self.catalog = getToolByName(context, "portal_catalog")
            self.mtool = getToolByName(self.context, "portal_membership")

            brains = []
            if keep_selection_order:
                brains = [uuidToCatalogBrain(uid) for uid in selection]
            else:
                brains = self.catalog(UID=selection, show_inactive=True)

            for brain in brains:
                if not brain:
                    continue

                # remove everyone so we know if we missed any
                selection.remove(brain.UID)
                obj = brain.getObject()

                # Check if item is allowed to be removed
                adapter = IDeleteProtected(obj, None)
                if adapter:
                    if adapter.delete_protection:
                        msgid = _(
                            "protected_item_delete",
                            default=(
                                "Item ${title} is protected against " "being deleted"
                            ),
                            mapping={"title": obj.Title()},
                        )
                        self.errors.append(msgid)
                        return self.message(list())

                if self.required_obj_permission and not self.mtool.checkPermission(
                    self.required_obj_permission, obj
                ):
                    self.errors.append(
                        PloneMessageFactory(
                            'Permission denied for "${title}"',
                            mapping={"title": self.objectTitle(obj)},
                        )
                    )
                    continue
                self.action(obj)

            self.finish()
            return self.message(selection)


class DeleteProtectedView(BrowserView):

    def __call__(self):
        # XXX: We cannot simply redirect from here
        # See more: ZPublisher/WSGIPublisher.py(141)_exc_view_created_response()
        self.request.response.setStatus(200)
        return self.__parent__()



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting

from plone.testing import z2

try:
    from plone.testing.zope import WSGI_SERVER_FIXTURE
except ImportError:
    from plone.testing.z2 import ZSERVER_FIXTURE as WSGI_SERVER_FIXTURE

class CollectivefolderprotectionLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import collective_folderprotection

        self.loadZCML(package=collective_folderprotection)
        self.loadZCML(package=collective_folderprotection, name="overrides.zcml")

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, "collective_folderprotection:test_fixture")

        # Create a manager user
        pas = portal["acl_users"]
        pas.source_users.addUser("manager", "manager", "manager")
        pas.portal_role_manager.doAssignRoleToPrincipal("manager", "Manager")

        # Create a contributor user
        pas = portal["acl_users"]
        pas.source_users.addUser("contributor", "contributor", "contributor")
        pas.portal_role_manager.doAssignRoleToPrincipal("contributor", "Contributor")


FIXTURE = CollectivefolderprotectionLayer()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name="collective_folderprotection:Integration"
)
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name="collective_folderprotection:Functional"
)
ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, WSGI_SERVER_FIXTURE),
    name="collective_folderprotection:Acceptance",
)


# -*- coding: utf-8 -*-
from datetime import datetime
from datetime import timedelta
import unittest

from zope.annotation import IAnnotations

from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

from plone.dexterity.utils import createContentInContainer

from collective_folderprotection.behaviors.interfaces import IPasswordProtected

from collective_folderprotection.config import HASHES_ANNOTATION_KEY
from collective_folderprotection.config import HASH_COOKIE_KEY

from collective_folderprotection.testing import INTEGRATION_TESTING


class TestPasswordProtect(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # Create a folderish protected
        self.folder = createContentInContainer(
            self.portal, "folderish_protected", title="Protected"
        )

    def test_not_enabled_by_default(self):
        passwordprotect = IPasswordProtected(self.folder)
        self.assertFalse(passwordprotect.is_password_protected())

    def test_not_allowed_if_cookie_not_present(self):
        passwordprotect = IPasswordProtected(self.folder)
        self.assertFalse(passwordprotect.allowed_to_access())

    def test_not_allowed_if_expired_hash(self):
        # Let's create a hash and put it in the cookie and the annotation
        pw_hash = "this-is-the-hash"
        ann = IAnnotations(self.folder)
        hashes = ann.get(HASHES_ANNOTATION_KEY, {})

        # We will put the time 1 day in the past, so it would be expired
        hashes[pw_hash] = datetime.now() - timedelta(days=1)

        ann[HASHES_ANNOTATION_KEY] = hashes

        self.request.response.setCookie(HASH_COOKIE_KEY, pw_hash)

        passwordprotect = IPasswordProtected(self.folder)
        self.assertFalse(passwordprotect.allowed_to_access())

    def test_allowed_if_valid_hash(self):
        # Let's create a hash and put it in the cookie and the annotation
        pw_hash = "this-is-the-hash"
        ann = IAnnotations(self.folder)
        hashes = ann.get(HASHES_ANNOTATION_KEY, {})

        # We will put the time 10 days in the future, so it would be valid
        hashes[pw_hash] = datetime.now() + timedelta(days=10)

        ann[HASHES_ANNOTATION_KEY] = hashes

        self.request.cookies[HASH_COOKIE_KEY] = pw_hash

        passwordprotect = IPasswordProtected(self.folder)
        self.assertTrue(passwordprotect.allowed_to_access())

    def test_assign_password(self):
        pw = "this-is-the-pw"
        passwordprotect = IPasswordProtected(self.folder)
        self.assertFalse(passwordprotect.is_password_protected())
        passwordprotect.assign_password(pw)
        self.assertTrue(passwordprotect.is_password_protected())

    def test_remove_password(self):
        pw = "this-is-the-pw"
        passwordprotect = IPasswordProtected(self.folder)
        passwordprotect.assign_password(pw)
        self.assertTrue(passwordprotect.is_password_protected())
        passwordprotect.remove_password()
        self.assertFalse(passwordprotect.is_password_protected())

    def test_set_password_on_creation(self):
        pw = "this-is-the-pw"

        protected = createContentInContainer(
            self.portal,
            "folderish_protected",
            title="Protected",
            passw_hash=pw,
        )

        passwordprotect = IPasswordProtected(protected)
        self.assertTrue(passwordprotect.is_password_protected())



# -*- coding: utf-8 -*-

from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "collective_folderprotection:uninstall",
        ]


def post_install(context):
    new_val = list()
    to_add = [
        "DeleteProtectionException",
        "PasswordProtectedUnauthorized",
        "RenameProtectionException",
    ]
    err_log = context.error_log
    for exc in err_log._ignored_exceptions:
        new_val.append(exc)

    for exc in to_add:
        if exc not in new_val:
            new_val.append(exc)

    err_log._ignored_exceptions = tuple(new_val)


def post_uninstall(context):
    new_val = list()
    to_remove = (
        "DeleteProtectionException",
        "PasswordProtectedUnauthorized",
        "RenameProtectionException",
    )
    err_log = context.error_log
    for exc in err_log._ignored_exceptions:
        if exc not in to_remove:
            new_val.append(exc)

    err_log._ignored_exceptions = tuple(new_val)



# -*- coding: utf-8 -*-


class DeleteProtected(object):
    def __init__(self, context):
        self.context = context

    def _get_delete_protection(self):
        result = False
        if self.context.delete_protection:
            result = True
        return result

    def _set_delete_protection(self, value):
        self.context.delete_protection = value

    delete_protection = property(_get_delete_protection, _set_delete_protection)



# -*- coding: utf-8 -*-
import logging
import six
from hashlib import md5
from datetime import datetime
from random import random

from z3c.form import button
from z3c.form import field
from z3c.form import form

from zope.annotation import IAnnotations

from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface

from DateTime.DateTime import DateTime

from plone.app.layout.icons.icons import CatalogBrainContentIcon
from plone.app.layout.icons.interfaces import IContentIcon
from plone.app.textfield.interfaces import IRichTextValue

from plone.z3cform.layout import wrap_form

from Products.Five.browser import BrowserView

from collective_folderprotection.behaviors.interfaces import IPasswordProtected
from collective_folderprotection.config import HASHES_ANNOTATION_KEY
from collective_folderprotection.config import HASH_COOKIE_KEY
from collective_folderprotection.config import TIME_TO_LIVE
from collective_folderprotection import _

logger = logging.getLogger(__name__)

class RenderPasswordView(BrowserView):
    def __call__(self):
        self.came_from = self.request.get("URL")

        self.request.set("came_from", self.came_from)
        # Get the object for which we need to get access to
        ob = getattr(self.__parent__, self.context.name)
        prompt = getMultiAdapter((ob, self.request), name="passwordprompt")
        return prompt()


class AskForPasswordView(BrowserView):
    """ """

    reason = ""

    def __call__(self):
        # This will be true if DX and pw-protected enabled
        context = self.context
        password_protected = IPasswordProtected(context, None)
        if not password_protected:
            # It could be that this is the default view for the
            # protected parent, so let's try that
            parent_dp = context.aq_parent.getDefaultPage()
            if parent_dp == context.id:
                context = context.aq_parent
                # This will be true if DX and pw-protected enabled
                password_protected = IPasswordProtected(context, None)

        if password_protected:
            reason = password_protected.passw_reason
            if reason:
                if IRichTextValue.providedBy(reason):
                    self.reason = reason.output_relative_to(context)
                else:
                    self.reason = reason

        if self.request.get("submit", False):
            # The password was submitted
            passw = self.request.get("password", "")
            if six.PY3:
                passw_hash = md5(passw.encode()).hexdigest()
            else:
                passw_hash = md5(passw).hexdigest()

            if password_protected:
                if password_protected.is_password_protected():
                    # If this is not true, means the Manager has not
                    # set a password for this resource yet, then do
                    # not authenticate...
                    # If there's no came_from, then just go to the
                    # object itself
                    came_from = self.request.get("came_from", context.absolute_url())
                    if passw_hash == password_protected.passw_hash:
                        # The user has entered a valid password, then
                        # we store a random hash with a TTL so we know
                        # he already authenticated
                        ann = IAnnotations(context)
                        hashes = ann.get(HASHES_ANNOTATION_KEY, {})
                        if six.PY3:
                            random_hash = md5(str(random()).encode()).hexdigest()
                        else:
                            random_hash = md5(str(random())).hexdigest()
                        while random_hash in hashes:
                            # This would be *REALLY* hard to happen,
                            # but just in case...
                            if six.PY3:
                                random_hash = md5(str(random()).encode()).hexdigest()
                            else:
                                random_hash = md5(str(random())).hexdigest()

                        hashes[random_hash] = datetime.now() + TIME_TO_LIVE
                        # Store the hash in the annotation
                        ann[HASHES_ANNOTATION_KEY] = hashes
                        if context is not self.context:
                            # Also store the hash in the default view
                            # object, if that's the case
                            ann = IAnnotations(self.context)
                            ann[HASHES_ANNOTATION_KEY] = hashes

                        # Save the hash in a cookie
                        path = context.getPhysicalPath()
                        virtual_path = self.request.physicalPathToVirtualPath(path)
                        options = {
                            "path": "/".join(("",) + virtual_path),
                            "expires": (DateTime("GMT") + 5).rfc822(),
                        }
                        logger.debug("Valid password, setting cookie")
                        self.request.response.setCookie(
                            HASH_COOKIE_KEY, random_hash, **options
                        )
                        # Now that we have our cookie set, go to
                        # the original url
                        self.request.response.redirect(came_from)
                        return
                    else:
                        # Invalid password, stay here, but mantain
                        # the "came_from"
                        self.request.set("came_from", came_from)

        self.request.response.setStatus(401, lock=True)
        return self.index()


def pw_protected(form):
    passw_behavior = IPasswordProtected(form.context)
    return passw_behavior.is_password_protected()


class AssignPasswordForm(form.Form):

    fields = field.Fields(IPasswordProtected)

    ignoreContext = False

    def updateActions(self):
        super(AssignPasswordForm, self).updateActions()
        if "remove_pw" in self.actions:
            self.actions["remove_pw"].addClass("btn-danger")
        if "save" in self.actions:
            self.actions["save"].addClass("btn-success")

    @button.buttonAndHandler(_("Save"), name="save")
    def save(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = _("Please correct errors")
            return
        passw = data.get("passw_hash", "")
        passw_reason = data.get("passw_reason", "")
        passw_behavior = IPasswordProtected(self.context)

        if passw and passw != "":
            passw_behavior.assign_password(passw)
            self.status = _("Password assigned.")

        if passw_reason:
            passw_behavior.passw_reason = passw_reason
        else:
            passw_behavior.passw_reason = ""
        self.request.response.redirect(self.context.absolute_url())

    @button.buttonAndHandler(_("Cancel"), name="cancel")
    def cancel(self, action):
        self.status = _("Cancelled.")
        self.request.response.redirect(self.context.absolute_url())

    @button.buttonAndHandler(
        _("Remove Password Protection"), name="remove_pw", condition=pw_protected
    )
    def remove_pw(self, action):
        passw_behavior = IPasswordProtected(self.context)
        passw_behavior.remove_password()
        self.status = _("Password protection removed from this location.")
        self.request.response.redirect(self.context.absolute_url())


AssignPasswordFormView = wrap_form(AssignPasswordForm)


class IAssignPasswordValidation(Interface):
    def allowed():
        """Decide when to show the Assign password tab"""


class AssignPasswordValidation(BrowserView):
    def allowed(self):
        obj_is_protected = False
        authorized = False

        # First we check if the current context has the password
        # protection enabled
        try:
            # This will be true if DX and pw-protected enabled
            IPasswordProtected(self.context)
            obj_is_protected = True
        except TypeError:
            pass

        if obj_is_protected:
            # Only allow to set password to Manager or Owner
            pm = self.context.portal_membership
            roles = pm.getAuthenticatedMember().getRolesInContext(self.context)
            if "Manager" in roles or "Owner" in roles:
                authorized = True

        return authorized


@implementer(IContentIcon)
class PasswordProtectedIcon(CatalogBrainContentIcon):
    def __init__(self, context, request, brain):
        self.context = context
        self.request = request
        self.brain = brain
        self.has_protection_enabled = False
        self.is_protected = False

        portal_types = self.context.portal_types
        portal_type = self.brain.portal_type
        behaviors = getattr(portal_types[portal_type], "behaviors", None)

        if (
            behaviors
            and "collective_folderprotection.behaviors.interfaces.IPasswordProtected"  # noqa: E501
            in behaviors
        ):
            self.has_protection_enabled = True
            ob = self.brain.getObject()
            passwordprotected = IPasswordProtected(ob)

            if passwordprotected.is_password_protected():
                self.is_protected = True

    @property
    def url(self):
        if not self.has_protection_enabled:
            return super(PasswordProtectedIcon, self).url

        if self.is_protected:
            path = "++resource++resources/lock_locked_16.png"
        else:
            path = "++resource++resources/lock_unlocked_16.png"

        portal_state_view = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        )
        portal_url = portal_state_view.portal_url()
        return "%s/%s" % (portal_url, path)

    @property
    def description(self):
        if not self.has_protection_enabled:
            return super(PasswordProtectedIcon, self).description

        if self.is_protected:
            return "%s protected by password." % self.brain["portal_type"]
        else:
            return "%s not protected by password." % self.brain["portal_type"]



# -*- coding: utf-8 -*-
import logging
import six
from datetime import datetime
from hashlib import md5

from zope.annotation import IAnnotations

from collective_folderprotection.config import HASHES_ANNOTATION_KEY
from collective_folderprotection.config import HASH_COOKIE_KEY

logger = logging.getLogger(__name__)

class PasswordProtected(object):
    def __init__(self, context):
        self.context = context

    def is_password_protected(self):
        return getattr(self.context, "passw_hash", False)

    def allowed_to_access(self):
        allowed = False
        request = self.context.REQUEST
        ann = IAnnotations(self.context)
        hashes = ann.get(HASHES_ANNOTATION_KEY, {})
        user_hash = request.cookies.get(HASH_COOKIE_KEY, None)
        logger.debug("Cookie hash: %s" % user_hash)
        logger.debug("Stored hashes: %s" % hashes.keys())

        if user_hash and user_hash in hashes:
            now = datetime.now()
            valid_until = hashes[user_hash]
            allowed = valid_until > now
        if allowed:
            logger.debug("User allowed to access protected resource")
        else:
            logger.debug("User blocked access to protected resource")
        return allowed

    def _assign_password(self, passw=None):
        ann = IAnnotations(self.context)

        if passw:
            # If there's a password, assign it
            if six.PY3:
                passw_hash = md5(passw.encode()).hexdigest()
            else:
                passw_hash = md5(passw).hexdigest()
            if HASHES_ANNOTATION_KEY in ann:
                # Remove old stored hashes
                del ann[HASHES_ANNOTATION_KEY]

            self.context.passw_hash = passw_hash
        else:
            # If no password was provided, just remove everything
            if HASHES_ANNOTATION_KEY in ann:
                # Remove old storde hashes
                del ann[HASHES_ANNOTATION_KEY]

            self.context.passw_hash = None

    def assign_password(self, passw):
        self._assign_password(passw)

    def remove_password(self):
        self._assign_password(None)

    def _get_password(self):
        return self.context.passw_hash

    def _set_password(self, value):
        if value:
            self.assign_password(value)

    def _get_passw_reason(self):
        return self.context.passw_reason

    def _set_passw_reason(self, value):
        self.context.passw_reason = value

    passw_hash = property(_get_password, _set_password)
    passw_reason = property(_get_passw_reason, _set_passw_reason)



# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from collective_folderprotection import _
from collective_folderprotection.behaviors.interfaces import IDeleteProtected
from collective_folderprotection.behaviors.interfaces import IPasswordProtected
from collective_folderprotection.behaviors.interfaces import IRenameProtected
from collective_folderprotection.exceptions import (
    PasswordProtectedUnauthorized,
    DeleteProtectionException,
    RenameProtectionException,
)

from OFS.event import ObjectWillBeMovedEvent
from Products.statusmessages.interfaces import IStatusMessage
from plone import api


def checkPassword(portal, request):
    portal_path = "/".join(portal.getPhysicalPath())
    # We get the full path
    if "VIRTUAL_URL_PARTS" in request:
        full_path = request.get("VIRTUAL_URL_PARTS")[1]
    else:
        full_path = request.get("PATH_INFO")

    if full_path.startswith(portal_path):
        # just strip the portal_path from the full_path and the '/'
        full_path = full_path[len(portal_path) + 1 :]
    ob = portal
    # Now iterate over each one
    for name in full_path.split("/"):
        authorized = False
        obj_is_protected = False
        try:
            ob = ob.unrestrictedTraverse(name)
        except Exception:
            # This path is not traversable or doesn't exist, just ignore
            break
        try:
            passwordprotected = IPasswordProtected(ob)
            obj_is_protected = True

        except TypeError:
            pass

        if obj_is_protected:
            # We are at a content type that is password protected, first check
            # if the password is actually set for this content, and then
            # see if we are not actually at the passwordprompt view
            if (
                passwordprotected.is_password_protected()
                and "passwordprompt" not in full_path
            ):
                # We are not at the passwordprotected prompt, so we now check
                # if the current user is not Manager or the Owner
                pm = portal.portal_membership
                roles = pm.getAuthenticatedMember().getRolesInContext(ob)
                if "Manager" in roles or "Owner" in roles:
                    # This user is the Owner or a Manager, so we just authorize
                    # it
                    authorized = True

                if not (authorized or passwordprotected.allowed_to_access()):
                    # User is not authorized to access this resource
                    raise PasswordProtectedUnauthorized(name=name)

    return None


def insertCheckPasswordHook(portal, event):
    """Add this hook to the post_traversal so we can check if some object
    during the traversal needs a password
    """
    try:
        event.request.post_traverse(checkPassword, (portal, event.request))
    except RuntimeError:
        pass


def preventRemove(object, event):
    # First check if the object itself is protected
    adapter = IDeleteProtected(object, None)
    request = api.env.getRequest()
    if adapter:
        if adapter.delete_protection:
            if request:
                messages = IStatusMessage(request)
                messages.add(
                    _("Item cannot be deleted because it is protected."), type="error"
                )
            raise DeleteProtectionException()
    else:
        # Check with the parent
        parent = aq_parent(object)
        adapter = IDeleteProtected(parent, None)
        if adapter:
            if adapter.delete_protection:
                if request:
                    messages = IStatusMessage(request)
                    messages.add(
                        _("Item cannot be deleted because its parent folder is protected."),
                        type="error",
                    )
                raise DeleteProtectionException()


def preventRename(object, event):
    # ObjectWillBeMovedEvent is a base class of ObjectWillBeAddedEvent, so we
    # use type() to be sure we only check ObjectWillBeMovedEvent
    if type(event) is not ObjectWillBeMovedEvent:
        return
    # Only check for items that remain in the same folder, with different
    # name (ie. renamed)
    request = api.env.getRequest()
    if event.oldParent is event.newParent and event.oldName != event.newName:
        # First check if the object itself is protected
        adapter = IRenameProtected(object, None)
        if adapter:
            if adapter.rename_protection:
                if request:
                    messages = IStatusMessage(request)
                    messages.add(
                        _("This item is protected against being renamed"), type="error"
                    )
                raise RenameProtectionException()
        else:
            # Check with the parent
            parent = aq_parent(object)
            adapter = IRenameProtected(parent, None)
            if adapter:
                if adapter.rename_protection:
                    if request:
                        messages = IStatusMessage(request)
                        messages.add(
                            _(
                                "This folder is protected against renaming "
                                "items inside of it."
                            ),
                            type="error",
                        )
                    raise RenameProtectionException()



# -*- coding: utf-8 -*-

from .. import _
from ..behaviors.interfaces import IRenameProtected
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.content.browser.actions import RenameForm as BaseRenameForm
from plone.app.content.browser.contents.rename import (
    RenameActionView as BaseRenameActionView,
)
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
try:
    from plone.base import PloneMessageFactory
except:
    from Products.CMFPlone import PloneMessageFactory


class RenameForm(BaseRenameForm):
    @button.buttonAndHandler(PloneMessageFactory("Rename"), name="Rename")
    def handle_rename(self, action):
        # First check if the object itself is protected
        protected = False
        adapter = IRenameProtected(self.context, None)
        if adapter:
            if adapter.rename_protection:
                IStatusMessage(self.request).add(
                    _("This item is protected against being renamed"),
                    type="error",
                )
                protected = True
        else:
            # Check with the parent
            parent = aq_parent(aq_inner(self.context))
            adapter = IRenameProtected(parent, None)
            if adapter:
                if adapter.rename_protection:
                    IStatusMessage(self.request).add(
                        _(
                            "This folder is protected against renaming "
                            "items inside of it."
                        ),
                        type="error",
                    )
                    protected = True
        if protected:
            self.request.response.redirect(self.context.absolute_url())
        else:
            return super(RenameForm, self).handle_rename(self, action)

    @button.buttonAndHandler(
        PloneMessageFactory("label_cancel", default="Cancel"), name="Cancel"
    )
    def handle_cancel(self, action):
        self.request.response.redirect(self.view_url())


class RenameActionView(BaseRenameActionView):
    def __call__(self):
        self.errors = list()
        context = aq_inner(self.context)
        catalog = getToolByName(context, "portal_catalog")

        for key in self.request.form.keys():
            if not key.startswith("UID_"):
                continue
            uid = self.request.form[key]
            brains = catalog.searchResults(UID=uid, show_inactive=True)
            if len(brains) == 0:
                continue
            obj = brains[0].getObject()

            # First check if the object itself is protected
            adapter = IRenameProtected(obj, None)
            if adapter:
                if adapter.rename_protection:
                    msgid = _(
                        "protected_item_rename",
                        default=("Item ${title} is protected against " "being renamed"),
                        mapping={"title": obj.Title()},
                    )
                    self.errors.append(msgid)
                    return self.message(list())
            else:
                # Check with the parent
                parent = aq_parent(aq_inner(obj))
                adapter = IRenameProtected(parent, None)
                if adapter:
                    if adapter.rename_protection:
                        self.errors.append(
                            _(
                                "This folder is protected against renaming "
                                "items inside of it."
                            )
                        )
                        return self.message(list())

        return super(RenameActionView, self).__call__()


class RenameProtectedView(BrowserView):

    def __call__(self):
        # XXX: We cannot simply redirect from here
        # See more: ZPublisher/WSGIPublisher.py(141)_exc_view_created_response()
        self.request.response.setStatus(200)
        return self.__parent__()



from setuptools import setup, find_packages
import os

version = '2.4.3'

long_description = (
    open('README.txt').read()
    + '\n' +
    open('CONTRIBUTORS.txt').read()
    + '\n' +
    open('CHANGES.txt').read()
    + '\n')

setup(name='collective_folderprotection',
      version=version,
      description="Provide delete, rename and password protection for Plone items.",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Framework :: Plone",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='folder protection plone',
      author='Enfold Systems, Inc.',
      author_email='info@enfoldsystems.com',
      url='https://github.com/collective/collective_folderprotection',
      license='gpl',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'six',
          'plone.app.contenttypes',
      ],
      extras_require={
          'test': [
              'cssselect',
              'lxml',
              'mock',
              'plone.api >=1.8.5',
              'plone.app.robotframework',
              'plone.app.testing [robot]',
              'plone.browserlayer',
              'plone.cachepurging',
              'plone.testing',
              'robotsuite',
              'testfixtures',
              'transaction',
              'tzlocal',
          ],
      },
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from collective_folderprotection.behaviors.interfaces import IPasswordProtected
from plone.app.layout.viewlets.common import ViewletBase
try:
    from plone.base.interfaces.siteroot import IPloneSiteRoot
except:
    from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot

class AlertViewlet(ViewletBase):
    """Displays alert messages when content is pw protected"""

    context_pw_protected = False
    folder_pw_protected = False
    parent_pw_protected = None

    def update(self):
        super(AlertViewlet, self).update()
        if self.request.response.getStatus() == 401:
            return
        is_default_page = False
        parent = None
        context = aq_inner(self.context)

        passw_behavior = IPasswordProtected(context, None)
        if passw_behavior:
            self.context_pw_protected = passw_behavior.is_password_protected()

        if not IPloneSiteRoot.providedBy(context):
            parent = aq_parent(context)
            if context.id == parent.getDefaultPage():
                is_default_page = True

        if is_default_page and not self.context_pw_protected:
            passw_behavior = IPasswordProtected(parent, None)
            if passw_behavior:
                self.context_pw_protected = passw_behavior.is_password_protected()

            if not IPloneSiteRoot.providedBy(parent):
                parent = aq_parent(parent)

        if parent:
            passw_behavior = IPasswordProtected(parent, None)
            if passw_behavior:
                self.folder_pw_protected = passw_behavior.is_password_protected()

            while not IPloneSiteRoot.providedBy(parent):
                parent = aq_parent(parent)
                passw_behavior = IPasswordProtected(parent, None)
                if passw_behavior and passw_behavior.is_password_protected():
                    self.parent_pw_protected = parent
                    break
